<?php

use Automattic\Jetpack\Constants;

defined('ABSPATH') || exit; // Exit if accessed directly

class WC_PayPlus_Gateway extends WC_Payment_Gateway_CC
{
	public $id = 'payplus-payment-gateway';
	public function __construct()
	{

		// class setup
		$this->has_fields         = false;
		$this->method_title       = __('PayPlus', 'payplus-payment-gateway');
		$this->method_description = __('PayPlus Credit Card Secure Payment', 'payplus-payment-gateway');
		$this->supports           = [
			'products',
			'subscriptions',
			'subscription_cancellation',
			'subscription_suspension',
			'subscription_reactivation',
			'subscription_amount_changes',
			'subscription_date_changes',
			'subscription_payment_method_change',
			'subscription_payment_method_change_customer',
			'subscription_payment_method_change_admin',
			'multiple_subscriptions',
			'refunds',
			'pre-orders',
			'tokenization',
		];

		// load assets
		add_action('admin_enqueue_scripts', [$this, 'load_admin_assets']);
		add_action('wp_enqueue_scripts', [$this, 'load_checkout_assets']);

		// detect support for multi language - wpml or polylang
		// $this->has_multilang = defined( 'ICL_LANGUAGE_CODE' );

		// load settings
		$this->init_form_fields();
		$this->init_settings();

		// define user set variables
		$this->title             = $this->get_option('title');
		$this->description       = $this->get_option('description');
		$this->api_test_mode     = $this->get_option('api_test_mode') == 'yes' ? true : false;
		$this->api_key           = $this->get_option('api_key');
		$this->secret_key 		 = $this->get_option('secret_key');
		$this->payment_page_id 	 = $this->get_option('payment_page_id');
		$this->rounding_decimals = $this->get_option('rounding_decimals');


		// - PayPlus Payment Options
		$this->hide_icon 			 = $this->get_option('hide_icon');
		$this->transaction_type 	 = $this->get_option('transaction_type');
		$this->initial_invoice 		 = $this->get_option('initial_invoice');
		$this->paying_vat 	 	 	 = $this->get_option('paying_vat');
		$this->paying_vat_iso_code	 = $this->get_option('paying_vat_iso_code');
		$this->foreign_invoices_lang = $this->get_option('foreign_invoices_lang');
		$this->send_products 		 = $this->get_option('send_products') == 'yes' ? true : false;

		if ($this->hide_icon == "no") {
			$this->icon = PAYPLUS_PLUGIN_URL . 'assets/images/payplus.png';
		}

		// - PayPlus Display Mode
		$this->display_mode  			= $this->get_option('display_mode');
		$this->iframe_height  			= $this->get_option('iframe_height');
		$this->import_applepay_script   = $this->get_option('import_applepay_script') == 'yes' ? true : false;

		// - PayPlus Settings Options
		$this->use_ipn     	 		  	= $this->get_option('use_ipn') == 'yes' ? true : false;
		$this->send_variations     	 	= $this->get_option('send_variations') == 'yes' ? true : false;
		$this->create_token     	  	= $this->get_option('create_token') == 'yes' ? true : false;
		$this->send_add_data 	  		= $this->get_option('send_add_data') == 'yes' ? true : false;
		$this->hide_identification_id 	= $this->get_option('hide_identification_id');
		$this->hide_payments_field 	  	= $this->get_option('hide_payments_field');
		$this->default_charge_method 	= $this->get_option('default_charge_method');
		$this->hide_other_charge_methods= $this->get_option('hide_other_charge_methods');
		$this->sendEmailApproval 	  	= $this->get_option('sendEmailApproval');
		$this->sendEmailFailure 	  	= $this->get_option('sendEmailFailure');
		$this->paying_vat 	  			= $this->get_option('paying_vat');
		$this->successful_order_status 	= $this->get_option('successful_order_status');
		$this->failure_order_status 	= $this->get_option('failure_order_status');
		$this->logging     	  			= $this->get_option('logging') == 'yes' ? true : false;
		$this->fire_completed     	  	= $this->get_option('fire_completed') == 'yes' ? true : false;

		$this->invoice_lang 			= $this->get_option('invoice_lang') == 'en' ? 'en' : '';

		/* if ( $this->has_multilang ) {
			include_once( PAYPLUS_PLUGIN_DIR . '/includes/wc_payplus_multilang.php' );
			$this->form_fields = payplus_multilang()->init_multi_lang_form_fields( $this->form_fields );
		} */

		if ($this->api_test_mode) $apiURL = 'https://restapidev.payplus.co.il/api/v1.0/';
		else $apiURL = 'https://restapi.payplus.co.il/api/v1.0/';

		$this->payment_url 		   = $apiURL . 'PaymentPages/generateLink';
		$this->ipn_url             = $apiURL . 'PaymentPages/ipn';
		$this->refund_url          = $apiURL . 'Transactions/RefundByTransactionUID';
		$this->response_url        = add_query_arg('wc-api', 'payplus_gateway', home_url('/'));
		$this->add_payment_res_url = add_query_arg('wc-api', 'payplus_add_payment', home_url('/'));

		// save admin options
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

		// receipt page for iframe
		add_action('woocommerce_receipt_' . $this->id, [$this, 'receipt_page']);

		// handle ipn response
		add_action('woocommerce_api_payplus_gateway', [$this, 'ipn_response']);
		add_action('woocommerce_api_callback_response', [$this, 'callback_response']);

		// handle add cc ipn response
		add_action('woocommerce_api_payplus_add_payment', [$this, 'add_payment_ipn_response']);
		add_action('woocommerce_api_response_callback', [$this, 'response_callback']);

		// Subscription Handler
		if (class_exists('WC_Subscriptions_Order')) add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'scheduled_subscription_payment'), 10, 2);

		// Message for SandBox if Active
		add_action('woocommerce_before_checkout_form', [$this, 'msg_checkout_code']);
	}


	public function load_checkout_assets()
	{
		if (is_checkout()) {
			wp_scripts()->registered[ 'wc-checkout' ]->src = PAYPLUS_PLUGIN_URL . 'assets/js/checkout.min.js';
		}

		wp_enqueue_style('alertifycss', '//cdn.jsdelivr.net/npm/alertifyjs@1.13.1/build/css/alertify.min.css', array(), false, 'all');

		wp_register_script('alertifyjs', '//cdn.jsdelivr.net/npm/alertifyjs@1.13.1/build/alertify.min.js', array('jquery'), false, true);
		wp_enqueue_script('alertifyjs');
	}

	// load admin assets
	public function load_admin_assets()
	{
		wp_enqueue_script('wc-payplus-gateway-admin', PAYPLUS_PLUGIN_URL . 'assets/js/admin.min.js', ['jquery'], PAYPLUS_VERSION, true);
	}

	// init form fields
	public function init_form_fields()
	{
		$this->form_fields = [
			'title'             => [
				'title'       => __('Title', 'payplus-payment-gateway'),
				'type'        => 'text',
				'description' => __('This controls the title which the user sees during checkout', 'payplus-payment-gateway'),
				'default'     => __('Pay with Debit or Credit Card', 'payplus-payment-gateway'),
				'desc_tip'    => true,
			],
			'description'       => [
				'title'   => __('Description', 'payplus-payment-gateway'),
				'type'    => 'textarea',
				'default' => __('Pay securely by Debit or Credit Card through PayPlus', 'payplus-payment-gateway')
			],
			'settings_title'      => [
				'title' => __('PayPlus API Settings', 'payplus-payment-gateway'),
				'type'  => 'title',
			],
			'api_test_mode'     => [
				'title' => __('Test mode', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Enable Sandbox Mode', 'payplus-payment-gateway'),
			],
			'api_key'         => [
				'title'       => __('API KEY', 'payplus-payment-gateway'),
				'type'        => 'text',
				'description' => __('PayPlus API Key you can find in your account under Settings', 'payplus-payment-gateway'),
				'default'     => '',
				'desc_tip'    => true,
			],
			'secret_key'         => [
				'title'       => __('SECRET KEY', 'payplus-payment-gateway'),
				'type'        => 'text',
				'description' => __('PayPlus Secret Key you can find in your account under Settings', 'payplus-payment-gateway'),
				'default'     => '',
				'desc_tip'    => true,
			],
			'payment_page_id'         => [
				'title'       => __('Payment Page UID', 'payplus-payment-gateway'),
				'type'        => 'text',
				'description' => __('Your payment page UID can be found under Payment Pages in your side menu in PayPlus account', 'payplus-payment-gateway'),
				'default'     => '',
				'desc_tip'    => true,
			],
			'params_title'      => [
				'title' => __('PayPlus Payment Options', 'payplus-payment-gateway'),
				'type'  => 'title',
			],
			'transaction_type'      => [
				'title'   => __('Transactions Type', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('Payment Page Default Setting', 'payplus-payment-gateway'),
					'1'   => __('Charge', 'payplus-payment-gateway'),
					'2'   => __('Authorization', 'payplus-payment-gateway')
				],
				'default' => '1',
			],

			'invoice_display_title'     => [
				'title' => __('Invoicing & Documents Settings', 'payplus-payment-gateway'),
				'type'  => 'title',
			],
			'send_products'     => [
				'title' => __('Hide Products To Invoices and Documents', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Hide Products To Invoices and Documents', 'payplus-payment-gateway')
			],
			'send_variations'     => [
				'title' => __('Send product variations among with products (for invoices)', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Send product variations among with products (for invoices)', 'payplus-payment-gateway')
			],
			'initial_invoice'      => [
				'title'   => __('Initial Invoice / Receipt', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('Payment Page Default Setting', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway'),
					'2'   => __('No', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'rounding_decimals'         => [
				'title'       => __('Rounding Product Decimals', 'payplus-payment-gateway'),
				'type'        => 'number',
				'default'     => 2
			],
			'paying_vat'      => [
				'title'   => __('Invoice For Foreign Customers', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('Paying VAT', 'payplus-payment-gateway'),
					'1'   => __('Exempt VAT', 'payplus-payment-gateway'),
					'2'   => __('Exempt VAT If Customer Billing ISO Country Is Different Than...', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'paying_vat_iso_code'     => [
				'title'   => __('Your Business VAT Registration Country ISO Code', 'payplus-payment-gateway'),
				'type'    => 'text',
				'default' => 'IL'
			],
			'foreign_invoices_lang'     => [
				'title'   => __('The language of the invoices or documents issued to foreign customers (assuming your invoicing company supports this language)', 'payplus-payment-gateway'),
				'type'    => 'text',
				'default' => 'HE'
			],
			'display_title'     => [
				'title' => __('Display Options', 'payplus-payment-gateway'),
				'type'  => 'title',
			],
			'display_mode'      => [
				'title'   => __('Display Mode', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'redirect' => __('Redirect', 'payplus-payment-gateway'),
					'iframe'   => __('iFrame', 'payplus-payment-gateway'),
					'samePageIframe'   => __('iFrame on the same page', 'payplus-payment-gateway'),
					'popupIframe'   => __('iFrame in a Popup', 'payplus-payment-gateway'),
				],
				'default' => 'redirect',
			],
			'iframe_height'     => [
				'title'   => __('iFrame Height', 'payplus-payment-gateway'),
				'type'    => 'number',
				'default' => 700,
			],
			'import_applepay_script'     => [
				'title' => __('Apple Pay', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Include Apple Pay Script in Iframe Mode (You have to join the service first)', 'payplus-payment-gateway'),
			],
			'api_title'         => [
				'title'       => __('Payment Page Options', 'payplus-payment-gateway'),
				'type'        => 'title',
				'description' => __('You can set your payment pages as you like to', 'payplus-payment-gateway'),
			],
			'hide_icon'           => [
				'title'   => __('Hide PayPlus Icon In The Checkout Page', 'payplus-payment-gateway'),
				'type'    => 'checkbox',
				'label'   => __('Hide PayPlus Icon In The Checkout Page', 'payplus-payment-gateway'),
				'default' => 'no'
			],
			'use_ipn'           => [
				'title'   => __('Use IPN Authentication from PayPlus Servers', 'payplus-payment-gateway'),
				'type'    => 'checkbox',
				'label'   => __('Use IPN Authentication from PayPlus Servers', 'payplus-payment-gateway'),
				'default' => 'yes'
			],
			'create_token'           => [
				'title'   => __('Saved Credit Cards', 'payplus-payment-gateway'),
				'type'    => 'checkbox',
				'label'   => __('Enable Payment via Saved Cards', 'payplus-payment-gateway'),
				'default' => 'no'
			],
			'send_add_data'           => [
				'title'   => __('Add Data Parameter', 'payplus-payment-gateway'),
				'type'    => 'checkbox',
				'label'   => __('Send Add Data Parameter When Payment Created', 'payplus-payment-gateway'),
				'default' => 'no'
			],
			'hide_identification_id'           => [
				'title'   => __('Hide Identification Field In Payment Form', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('Payment Page Default Setting', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway'),
					'2'   => __('No', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'hide_payments_field'           => [
				'title'   => __('Hide Payments Number In Payment Form', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('Payment Page Default Setting', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway'),
					'2'   => __('No', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'hide_other_charge_methods'           => [
				'title'   => __('Hide other payment types on payment page', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('No', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway'),
				],
				'default' => '0',
			],
			'order_status_title'   => [
				'title' => __('Order Settings', 'payplus-payment-gateway'),
				'type'  => 'title',
			],
			'successful_order_status'      => [
				'title'   => __('Successful Order Status', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => wc_get_order_statuses(),
				'default' => 'wc-processing'
			],
			'fire_completed'      => [
				'title' => __('Payment Completed', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Fire Payment Completed On Successful Charge', 'payplus-payment-gateway'),
				'default' => 'yes'
			],
			'failure_order_status'      => [
				'title'   => __('Failure Order Status', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => wc_get_order_statuses(),
				'default' => 'wc-on-hold'
			],
			'sendEmailApproval' => [
				'title'   => __('Successful Transaction E-mail Through PayPlus Servers', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('No', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'sendEmailFailure' => [
				'title'   => __('Failure Transaction E-mail Through PayPlus Servers', 'payplus-payment-gateway'),
				'type'    => 'select',
				'options' => [
					'0'   => __('No', 'payplus-payment-gateway'),
					'1'   => __('Yes', 'payplus-payment-gateway')
				],
				'default' => '0',
			],
			'logging'      => [
				'title' => __('Logging', 'payplus-payment-gateway'),
				'type'  => 'checkbox',
				'label' => __('Log debug messages', 'payplus-payment-gateway'),
				'default' => 'yes'
			]
		];
	}


	public function process_refund($order_id, $amount = NULL, $reason = '')
	{
		$order = wc_get_order($order_id);
		if ($this->logging) $logger = wc_get_logger();
		$refunded_amount = round((float)$order->get_meta('payplus_total_refunded_amount'), 2);
		if (!$refunded_amount or $refunded_amount <= 0) $refunded_amount = 0;
		$transaction_uid = $order->get_meta('payplus_transaction_uid');
		if ($order->get_meta('payplus_type') != "Charge") {
			if ($this->logging) $logger->add('payplus_process_refund', 'Already Charged or Original Transaction Are Not J5');
			$order->add_order_note(sprintf(__('PayPlus Refund is Failed<br />You cannot refund transaction that not charged. Current Transaction Type Status: %s'), $order->get_meta('payplus_type')));
			return false;
		}
		if (round((float)$refunded_amount, 2) >= round((float)$order->get_total(), 2)) {
			if ($this->logging) $logger->add('payplus_process_refund', 'You Cannot charge more then the original transaction amount');
			$order->add_order_note(sprintf(__('PayPlus Refund is Failed<br />You cannot refund more then the refunded total amount. Refunded Amount: %s %s'), $refunded_amount, $order->get_currency()));
			return false;
		}

		// Sending Products
		$refundByItems = false;

		if (!$this->send_products) {
			if ($amount == $order->get_total() and $refunded_amount == 0) {
				$refundByItems = true;
				if ($this->logging) $logger->add('payplus_process_refund', 'Sending Items...');

				$totalCartAmount = 0;
				foreach ($order->get_items(['line_item', 'fee', 'coupon']) as $item => $item_data) {
					$item_meta_name = '';
					$couponPrice = 0;
                    $quantity = ($item_data['quantity'] ? $item_data['quantity'] : '1');

					// Only for product variation
					$item_meta = @new WC_Order_Item_Meta($item_data);
					if ($meta = $item_meta->display(true, true)) $item_meta_name = $meta;

					if ($item_data['type'] == "coupon") {
						if ($item_data['discount']) $couponPrice = $item_data['discount'] + $item_data['discount_tax'];
							else $couponPrice = $item_data['line_total'] + $item_data['line_total_tax'];
						$productPrice = '-' . round($couponPrice, $this->rounding_decimals);
						$totalCartAmount += ($productPrice);
					} else if ($item_data['type'] == "fee") {
						$productPrice = round(($item_data['line_total'] + $item_data['line_total_tax']), $this->rounding_decimals);
						$totalCartAmount += ($productPrice);
					} else {
						$productPrice = round(($item_data['line_subtotal'] + $item_data['line_subtotal_tax']) / $item_data['quantity'], $this->rounding_decimals);
						$quantity = $item_data['quantity'];
						$totalCartAmount += ($productPrice * $quantity);
					}

					$product = new WC_Product($item_data['product_id']);
					$productSKU = $product->get_sku();

					$productsItems[] = '{
							"name" : "' . str_replace(["'", '"', "\n","\\"], '', strip_tags($item_data['name'])) . '",
							' . ($item_meta_name && $this->send_variations ? '"product_invoice_extra_details" : "' . str_replace(["'", '"', "\n","\\"], '', strip_tags($item_meta_name)) . '",' : '') . '
							"barcode": "' . ($productSKU ? $productSKU : $item_data['product_id']) . '",
							"quantity" : ' . ($quantity ? $quantity : '1') . ',
							"price" : ' . $productPrice . '
							}';
				}

				// order shipping
				if ($order->get_shipping_total()) {
					foreach ($order->get_shipping_methods() as $shipping_method) {
						$shipping_method_data = $shipping_method->get_data();
						$totalLine = round($shipping_method_data['total'], $this->rounding_decimals);
						$productsItems[] = '{	
							"name" : "' . __('Shipping', 'payplus-payment-gateway') . ' - ' . str_replace(["'", '"',"\\"], '', $shipping_method_data['name']) . '",
							"quantity" : 1,
							"price" : ' . $totalLine . '
							}';
						$totalCartAmount += $totalLine;
					}
				}
			}
			/* if ($total_discount = $order->get_discount_total()) {
				$disCountPrice = round($total_discount + $order->get_discount_tax(), $this->rounding_decimals);
				$productsItems[] = '{
						"name" : "' . __('Discount', 'payplus-payment-gateway') . '",
						"quantity" : 1,
						"price" : -' . $disCountPrice . '
						}';
				$totalCartAmount -= $disCountPrice;
			} */
		}

		$payload = '{
				"transaction_uid": "' . $transaction_uid . '",
				"amount": "' . $amount . '",
				' . ($refundByItems && !$this->send_products ? '
				"items": [
					' . implode(",", $productsItems) . '
					]' : '
				"more_info" : "' . __('Refund for Order Number: ', 'payplus-payment-gateway') . $order_id . '"
					') . '
				}';

		$response = $this->post_payplus_ws($this->refund_url, $payload);
		$res = json_decode(wp_remote_retrieve_body($response));
		if ($res->results->status == "success" && $res->data->transaction->status_code == "000") {
			update_post_meta($order_id, 'payplus_total_refunded_amount', round($refunded_amount + $amount, 2));
			$order->add_order_note(sprintf(__('PayPlus Refund is Successful<br />Refund Transaction Number: %s<br />Amount: %s %s<br />Reason: %s', 'payplus-payment-gateway'), $res->data->transaction->number, $res->data->transaction->amount, $order->get_currency(), $reason));
			if ($this->logging) $logger->add('payplus_process_refund', 'WP Remote Post Completed: ' .  print_r($res, true));
			return true;
		} else {
			$order->add_order_note(sprintf(__('PayPlus Refund is Failed<br />Status: %s<br />Description: %s', 'payplus-payment-gateway'), $res->results->status, $res->results->description));
			if ($this->logging) $logger->add('payplus_process_refund', 'WP Remote Post Error: ' . print_r($res, true));
			return false;
		}
	}

	// custom option page html
	public function admin_options()
	{
		$title  = __('PayPlus', 'payplus-payment-gateway');
		$desc   = __('For more information about PayPlus and Plugin versions <a href="https://www.payplus.co.il/wordpress" target="_blank">www.payplus.co.il/wordpress</a>', 'payplus-payment-gateway');
		$credit = __('This plugin was developed by <a href="https://www.payplus.co.il">PayPlus LTD</a>', 'payplus-payment-gateway');
		ob_start();
		$this->generate_settings_html();
		$settings = ob_get_clean();

		echo "<div id='payplus-options'>
			<h3>$title</h3>
			<p>$desc</p>
			<table class='form-table'>$settings</table>
			<div class='payplus-credit' style='position: absolute; bottom: 33px;'>$credit</div>
		</div>";
	}

	// remove credit card form from checkout
	public function payment_fields()
	{
		if ($this->supports('tokenization') && is_checkout()) {
			$this->tokenization_script();
			$this->saved_payment_methods();
			$this->save_payment_method_checkbox();
		}
	}

	// method description and add cc to account text
	public function save_payment_method_checkbox()
	{
		$html = '<div class="payplus-option-description-area">';
		if ($this->description) {
			$html .= '<p class="form-row payment-method-description">' . $this->description . '</p>';
		}
		if ($this->create_token && $this->id) {
			$html .= sprintf(
				'<p class="form-row woocommerce-SavedPaymentMethods-saveNew">
					<input id="wc-%1$s-new-payment-method" name="wc-%1$s-new-payment-method" type="checkbox" value="true" style="width:auto;" />
					<label for="wc-%1$s-new-payment-method" style="display:inline;">%2$s</label>
				</p>',
				esc_attr($this->id),
				esc_html__('Save credit card in my account', 'payplus-payment-gateway')
			);
		}
		$html .= '</div>';
		$html .= "<iframe id='pp_form' name='payplus-iframe' style='display:none; width: 100%; height: " . $this->iframe_height . "px; border: 0;'></iframe>";

		echo apply_filters('woocommerce_payment_gateway_save_new_payment_method_option_html', $html, $this);
	}

	// redirect to receipt page (payment page)
	public function process_payment($order_id)
	{
		$order    = wc_get_order($order_id);
		$is_token = (isset($_POST['wc-'.$this->id.'-payment-token']) && $_POST['wc-'.$this->id.'-payment-token'] !== 'new') ? true : false;
		$order->delete_meta_data("save_payment_method");
		$order->add_meta_data("save_payment_method",isset($_POST['wc-'.$this->id.'-new-payment-method']) ? '1':'0');
		$order->save_meta_data();
		$redirect_to = add_query_arg('order-pay', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))));
		if ($this->logging) $logger = wc_get_logger();

		if ($is_token) {
			$token_id = wc_clean($_POST['wc-'.$this->id.'-payment-token']);
			$token    = WC_Payment_Tokens::get($token_id);
			if (!$this->checkValidateCard($token)) {
				if ($this->logging) $logger->add('payplus_payment_using_token', 'Token Expired: ' . $token);
				wc_add_notice(__('Error: user or other, please contact PayPlus support', $this->id), 'error');
				return;
			}
			$response = $this->receipt_page($order_id, $token);
			if ($this->logging) $logger->add('payplus_payment_using_token', 'RestAPI Response: ' . print_r($response, true));

			if ($response->data->status == "approved" && $response->data->status_code == "000" && $response->data->transaction_uid) {
				$this->updateMetaData($order_id, $response);
				if ($response->data->type == "Charge") {
					if ($this->fire_completed) $order->payment_complete();
					$order->update_status($this->successful_order_status);
				} else {
					$order->update_status('wc-on-hold');
				}
				$order->add_order_note(sprintf(__('PayPlus Token Payment Successful<br/>Transaction Number: %s', 'payplus-payment-gateway'), $response->data->number));
				if ($this->logging) {
					$logger->add('payplus_payment_using_token', 'WP Remote Post Completed');
					$logger->add('payplus_payment_using_token', 'Response: ' . print_r($response, true));
				}
			} else {
				if ($this->logging) $logger->add('payplus_payment_using_token', 'WP Remote Post Error: ' . print_r($response, true));
				$order->add_order_note(sprintf(__('PayPlus Token Payment Failed<br/>Transaction Number: %s', 'payplus-payment-gateway'), $response->data->number));
				$order->update_status($this->failure_order_status);
				wc_add_notice(sprintf(__('Error: credit card declined: %s', 'payplus-payment-gateway'), print_r($response, true)), 'error');
				return;
			}
		}

		$result = [
			'result'   => 'success',
			'redirect' => $redirect_to,
			'viewMode' => $this->display_mode
		];
		if (in_array($this->display_mode, ['samePageIframe', 'popupIframe'])) {
			$result['payplus_iframe'] = $this->receipt_page($order_id, null, false, '', 0, true);
		}
		return $result;
	}

	public function checkValidateCard($token = NULL)
	{
		if (!$token->get_token()) return false;
		if ($token->get_expiry_year() > date("Y")) return true;
		if ($token->get_expiry_year() < date("Y")) return false;
		if ($token->get_expiry_month() >= date("m")) return true;
	}

	// receipt page - iframe or redirect
	public function receipt_page($order_id, $token = null, $subscription = false, $custom_more_info = '', $subscription_amount = 0, $inline = false)
	{
		$order = wc_get_order($order_id);
		$logger = wc_get_logger();
		if ($this->logging) {
			$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'New Payment Process Fired');
			if ($token) $logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'Token has been used');
			if ($subscription && !$token) $logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), '--- Token is empty or invalid - exit ---');
		}
		if ($subscription && !$token) return false;
		// order items
		
		$totalCartAmount = 0;
		if (!$this->send_products) {
			foreach ($order->get_items(['line_item', 'fee', 'coupon']) as $item => $item_data) {
				$item_name = $item_data['name'];
				$item_meta_name = '';
				$couponPrice = 0;
                $quantity = ($item_data['quantity'] ? $item_data['quantity'] : '1');

				// Only for product variation
				$item_meta = @new WC_Order_Item_Meta($item_data);
				if ($meta = $item_meta->display(true, true)) $item_meta_name = $meta;

				if ($item_data['type'] == "coupon") {
					if ($item_data['discount']) $couponPrice = $item_data['discount'] + $item_data['discount_tax'];
						else $couponPrice = $item_data['line_total'] + $item_data['line_total_tax'];
					$productPrice = '-' . round($couponPrice, $this->rounding_decimals);
					$totalCartAmount += ($productPrice);
				} else if ($item_data['type'] == "fee") {
					$productPrice = round(($item_data['line_total'] + $item_data['line_total_tax']), $this->rounding_decimals);
					$totalCartAmount += $productPrice;
				} else {
					$productPrice = round(($item_data['line_subtotal'] + $item_data['line_subtotal_tax']) / $item_data['quantity'], $this->rounding_decimals);
					$totalCartAmount += ($productPrice * $quantity);
				}

				$product = new WC_Product($item_data['product_id']);
				$productSKU = $product->get_sku();

				$productsItems[] = '{	
					"name" : "' . str_replace(["'", '"', "\n","\\"], '', strip_tags($item_name)) . '",
					' . ($item_meta_name && $this->send_variations ? '"product_invoice_extra_details" : "' . str_replace(["'", '"', "\n","\\"], '', strip_tags($item_meta_name)) . '",' : '') . '
					"barcode": "' . ($productSKU ? $productSKU : $item_data['product_id']) . '",
					"quantity" : ' . ($quantity ? $quantity : '1') . ',
					"price" : ' . $productPrice . '
					}';
			}
			// order shipping
			if ((version_compare(Constants::get_constant('WC_VERSION'), '3.0', '<') ? $order->get_total_shipping() : $order->get_shipping_total())) {
				foreach ($order->get_shipping_methods() as $shipping_method) {
					$shipping_method_data = $shipping_method->get_data();
					$totalLine = round((version_compare(Constants::get_constant('WC_VERSION'), '3.0', '<') ? $order->get_total_shipping() : $order->get_shipping_total()), $this->rounding_decimals);
					$productsItems[] = '{	
					"name" : "' . __('Shipping', 'payplus-payment-gateway') . ' - ' . str_replace(["'", '"',"\\"], '', $shipping_method_data['name']) . '",
					"quantity" : 1,
					"price" : ' . $totalLine . '
					}';
					$totalCartAmount += $totalLine;
				}
			}
			/* if ($total_discount = $order->get_discount_total()) {
			
				$disCountPrice = round($total_discount + $order->get_discount_tax(), $this->rounding_decimals);
				$productsItems[] = '{	
					"name" : "' . __('Discount', 'payplus-payment-gateway') . '",
					"quantity" : 1,
					"price" : -' . $disCountPrice . '
					}';
				$totalCartAmount -= $disCountPrice;
			} */
		}
		$langCode = explode("_", get_locale());

		/* if ($subscription_amount) {
			$productsItems = Array();
			$totalCartAmount=0;
			$productsItems[] = '{	
					"name" : "'.str_replace( [ "'", '"' ], '', __('Subscription Renewal: ').$order_id).'",
					"barcode": "subrenew667165",
					"quantity" : 1,
					"price" : '.$subscription_amount.'
					}';
				$totalCartAmount = $subscription_amount;
		} */
		$setInvoice = '';
		$payingVat = '';
		$invoiceLanguage = '';
		$addChargeLine = '';
		$cell_phone = str_replace(["'", '"',"\\"], '', $order->get_billing_phone());
		$address = trim(str_replace(["'", '"',"\\"], '', $order->get_billing_address_1() . ' ' . $order->get_billing_address_2()));
		$city = str_replace(["'", '"',"\\"], '', $order->get_billing_city());
		$postal_code = str_replace(["'", '"',"\\"], '', $order->get_billing_postcode());
		$customer_country_iso = $order->get_billing_country();
		if ($subscription) {
			$addChargeLine = '"charge_method": 1,';
		} else if ($this->settings['transaction_type'] != "0") {
			$addChargeLine = '"charge_method": ' . $this->settings['transaction_type'] . ',';
		}

		// Invoice Initial
		if ($this->initial_invoice == "1") $setInvoice = '"initial_invoice": true,';
		else if ($this->initial_invoice == "2") $setInvoice = '"initial_invoice": false,';

		// Paying Vat & Invoices
		if ($this->paying_vat == "0") $payingVat = '"paying_vat": true,';
		else if ($this->paying_vat == "1") $payingVat = '"paying_vat": false,';
		else if ($this->paying_vat == "2") {
			if (trim(strtolower($customer_country_iso)) != trim(strtolower($this->paying_vat_iso_code))) {
				$payingVat = '"paying_vat": false,';
				$invoiceLanguage = '"invoice_language": "' . strtolower($this->foreign_invoices_lang) . '",';
			} else $payingVat = '"paying_vat": true,';
		}
		$this->default_charge_method = ($this->default_charge_method) ?:'credit-card';
		$bSaveToken = (bool)($order->get_meta("save_payment_method") === '1');
		$hideOtherChargeMethods = (isset($this->hide_other_charge_methods) && $this->hide_other_charge_methods === '1') ? 'true':'false';
		$payload = '{
    "payment_page_uid": "' . $this->settings['payment_page_id'] . '",
	' . $addChargeLine . '
    "expiry_datetime": "30",
    "hide_other_charge_methods": '. $hideOtherChargeMethods .',
    "language_code": "' . trim(strtolower($langCode[0])) . '",
    "refURL_success": "' . $this->response_url . '",
    "refURL_failure": "' . $this->response_url . '",
    "refURL_callback": "'. get_site_url(null,'/?wc-api=callback_response') .'",
	"charge_default":"' . $this->default_charge_method . '",
	' . $payingVat . '
    "customer": {
        "customer_name":"' . str_replace(["'", '"',"\\"], '', ($order->get_billing_company() ? $order->get_billing_company() : $order->get_billing_first_name() . ' ' . $order->get_billing_last_name())) . '",
        "email":"' . $order->get_billing_email() . '",
		"cell_phone": ' . ($cell_phone ? '"' . $cell_phone . '"' : 'null') . ',
		"address": ' . ($address ? '"' . $address . '"' : 'null') . ',
		"city": ' . ($city ? '"' . $city . '"' : 'null') . ',
		"postal_code": ' . ($postal_code ? '"' . $postal_code . '"' : 'null') . ',
		"country_iso": ' . ($customer_country_iso ? '"' . $customer_country_iso . '"' : 'null') . '
    }' . (!$this->send_products ? ',
    "items": [
        ' . implode(",", $productsItems) . '
    ]' : '') . ',
	' . ($token ? '"token" : "' . (is_object($token) ? $token->get_token() : $token) . '",' : '') . '
    "amount": ' . ($this->send_products ? $order->get_total() : round($totalCartAmount, $this->rounding_decimals)) . ',
    "currency_code": "' . $order->get_currency() . '",
    "sendEmailApproval": ' . ($this->sendEmailApproval == 1 ? 'true' : 'false') . ',
    "sendEmailFailure": ' . ($this->sendEmailFailure == 1 ? 'true' : 'false') . ',
    "create_token": '. ($bSaveToken ? 'true':'false') .',
    ' . $setInvoice . '
	' . $invoiceLanguage . '
    ' . ($this->send_add_data ? '"add_data": "' . $order_id . '",' : '') . '
    ' . ($this->hide_payments_field > 0 ? '"hide_payments_field": ' . ($this->hide_payments_field == 1 ? 'true' : 'false') . ',' : '') . '
    ' . ($this->hide_identification_id > 0 ? '"hide_identification_id": ' . ($this->hide_identification_id == 1 ? 'true' : 'false') . ',' : '') . '
	"more_info": "' . ($custom_more_info ? $custom_more_info : $order_id) . '"
}';
		if ($this->logging) {
			$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'Payload data before Sending to PayPlus');
			$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'Request: ' . print_r($payload, true));
		}

		$response = $this->post_payplus_ws($this->payment_url, $payload);
		$res = json_decode(wp_remote_retrieve_body($response));
		if ($this->logging) {
			$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'WS PayPlus Response');
			$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'Response: ' . print_r($res, true));
		}
		if ($token || $inline) return $res;
		if (isset($res->data->payment_page_link) && $this->validateUrl($res->data->payment_page_link)) {
			if ($this->logging) $logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'WS Redirecting to Page: ' . $res->data->payment_page_link);
			$this->get_payment_page($res->data->payment_page_link);
		} else {
			if ($this->logging) {
				$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'WP Remote Post Error: ' . print_r($res, true));
				$logger->add('payplus_process_payment' . ($subscription ? '_subscription' : ''), 'WP Remote Post Message: ' . print_r($response['body'], true));
			}
			echo __('Something went wrong with the payment page') . '<hr /><b>Error:</b> ' . print_r($response['body'], true);
			// wp_die();
		}
	}

	// get payment page - iframe or redirect
	public function get_payment_page($res)
	{
		if (!$this->display_mode || $this->display_mode == 'default') {
			$mainPluginOptions = get_option('woocommerce_payplus-payment-gateway_settings' );
			$this->display_mode = ($mainPluginOptions['display_mode'] ?:'redirect');
			$this->iframe_height = ($this->iframe_height ?:700);
		}
		if ($this->display_mode == 'iframe') {
			echo "<form name='pp_form' target='payplus-iframe' method='GET' action='" . $res . "'></form>";
			echo "<iframe id='pp_form' name='payplus-iframe' style='width: 100%; height: " . $this->iframe_height . "px; border: 0;'></iframe>";
			if ($this->import_applepay_script) echo '<script src="https://payments' . ($this->api_test_mode ? 'dev' : '') . '.payplus.co.il/statics/applePay/script.js" type="text/javascript"></script>';
		} else {
			echo "<form name='pp_form' method='GET' action='" . $res . "'></form>";
		}
		echo '<script type="text/javascript"> document.pp_form.submit(); </script>';
	}

	public function validateUrl($url = NULL)
	{
		if (filter_var($url, FILTER_VALIDATE_URL) === FALSE) return false;
		else return true;
	}

	public function post_payplus_ws($url, $payload)
	{

		$args = array(
			'body' => $payload,
			'timeout'     => '5',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking'    => true,
			'headers'     => [],
			'headers' => array(
				'Content-Type' => 'application/json',
				'Authorization' => '{"api_key":"' . $this->api_key . '","secret_key":"' . $this->secret_key . '"}'
			)
		);
		$response = wp_remote_post($url, $args);
		return $response;
	}

	public function ipn_response()
	{

		$REQUEST = $this->arr_clean($_REQUEST);
		$data = [
			'transaction_uid'=>$REQUEST['transaction_uid'],
			'voucher_id'=>$REQUEST['voucher_num'],
			'token_uid'=>$REQUEST['token_uid'],
			'type'=>$REQUEST['type'],
			'order_id'=>$REQUEST['more_info'],
			'status_code'=>$REQUEST['status_code'],
			'number'=>$REQUEST['number'],
			'expiry_year'=>$REQUEST['expiry_year'],
			'expiry_month'=>$REQUEST['expiry_month'],
			'four_digits'=>$REQUEST['four_digits'],
			'brand_id'=>$REQUEST['brand_id'],
		];
		$order = $this->validateOrder($data);
		WC()->session->__unset('save_payment_method');
		wp_redirect($this->get_return_url($order));
	}

	// check ipn response
	public function callback_response(){
		$json = file_get_contents('php://input');
		$response = json_decode($json,true);
		$inputData['transaction_uid'] 	= $response['transaction']['uid'];
		$inputData['voucher_id']= $response['transaction']['voucher_number'];
		$inputData['token_uid']= $response['data']['card_information']['token'];
		$inputData['type']= $response['transaction_type'];
		$inputData['order_id'] =       $response['transaction']['more_info'];
		$inputData['status_code'] =       $response['transaction']['status_code'];
		$inputData['number'] =       $response['transaction']['number'];
		$inputData['expiry_year'] =       $response['data']['card_information']['expiry_year'];
		$inputData['expiry_month'] =       $response['data']['card_information']['expiry_month'];
		$inputData['four_digits'] =       $response['data']['card_information']['four_digits'];
		$inputData['brand_id'] =       $response['data']['card_information']['brand_id'];
		$this->validateOrder($inputData);
	}

	protected function validateOrder($data) {

		if ($this->logging) {
			$logger = wc_get_logger();
			$logger->add('payplus_ipn', 'New IPN Fired');
			$logger->add('payplus_ipn', 'Result: ' . print_r($data, true));
		}

		if (!isset($data['voucher_id']) || !isset($data['transaction_uid'])) {
			if ($this->logging){
				$logger->add('payplus_ipn', 'IPN Error: missing Order ID, missing Voucher Number / Transaction UID');
			}
			return;
		}
		$transaction_uid 	= $data['transaction_uid'];
		$token_uid      = $data['token_uid'];
		$type    		= $data['type'];
		$order_id = trim($data['order_id']);
		$userID = 0;
		$order    = wc_get_order($order_id);
		if ($order->get_user()!= false) {
			$userID = $order->get_user_id();
		}
		$IPNCheck = false;
		$createToken = false;
		$bSaveToken = ($order->get_meta('save_payment_method') === '1');
		if ($bSaveToken === true && $userID) {
			$createToken = true;
		}

		if ($order->get_meta('order_validated') == '1') {
			return $order;
		}

		if ($this->use_ipn) {
			$payload = '{
				"transaction_uid": "' . $transaction_uid . '"
					}';

			if ($this->logging) $logger->add('payplus_ipn', 'IPN Request: ' . print_r($payload, true));

			$response = $this->post_payplus_ws($this->ipn_url, $payload);
			$res = json_decode(wp_remote_retrieve_body($response));
			if ($res->results->status == "error") {
				if ($this->logging) $logger->add('payplus_ipn', 'Error IPN Error: ' . print_r($res, true));
				$order->update_status($this->failure_order_status);
				$order->add_order_note(sprintf(__('PayPlus IPN Failed<br/>Transaction UID: %s', 'payplus-payment-gateway'), $transaction_uid));
			} else {
				if ($res->data->status == "approved" && $res->data->status_code == "000" && $res->results->status == "success") $IPNCheck = true;
			}
			
		}
		if (($this->use_ipn && $IPNCheck) || (!$this->use_ipn && $data['status_code'] == "000")) {
			
			if ($this->logging) $logger->add('payplus_ipn', 'Success IPN Result: ' . print_r($data, true));
			if ($this->create_token && $token_uid && $userID && $createToken === true) {
				$this->save_token($data, $userID);
			}
			if ($order->get_user_id() > 0) update_user_meta($order->get_user_id(), 'cc_token', $data['token_uid']);
			$this->updateMetaData($order_id, $data);
			if ($type == "Charge") {
				if ($this->fire_completed) $order->payment_complete();
				$order->update_status($this->successful_order_status);
			} else $order->update_status('wc-on-hold');
			
			if ($type == "Approval") {
				$order->add_order_note(sprintf(__('PayPlus Pre-Authorization is Successful<br/>Transaction Number: %s', 'payplus-payment-gateway'), $data['number']));
			} else {
				$order->add_order_note(sprintf(__('PayPlus Payment Successful<br/>Transaction Number: %s', 'payplus-payment-gateway'), $data['number']));
			}
		} else {
			$order->update_status($this->failure_order_status);
			$order->add_order_note(__('PayPlus payment failed', 'payplus-payment-gateway'));
		}
		$order->add_meta_data('order_validated',1);
		$order->save_meta_data();
		return $order;
	}


	public function updateMetaData($order_id, $response)
	{
		$appVars = array('type', 'method', 'number', 'status', 'status_code', 'status_description', 'currency', 'four_digits', 'expiry_month', 'expiry_year', 'number_of_payments', 'first_payment_amount', 'rest_payments_amount', 'voucher_num', 'approval_num', 'transaction_uid', 'token_uid', 'more_info', 'alternative_method_id', 'add_data', 'customer_name', 'identification_number', 'brand_name', 'alternative_method_name', 'credit_terms', 'secure3D_tracking');
		for ($i = 0; $i < count($appVars); $i++) {
			unset($value);
			if (is_object($response)) {
				if (isset($response->data->{$appVars[$i]})) $value = $response->data->{$appVars[$i]};
				else continue;
			} else {
				if (isset($response[$appVars[$i]])) $value = $response[$appVars[$i]];
				else continue;
			}
			update_post_meta($order_id, 'payplus_' . $appVars[$i], wc_clean($value));
		}
		return;
	}

	// save token to user
	public function save_token($res, $user_id = false)
	{
		if ($this->logging) {
			$logger = wc_get_logger();
			if ($this->logging){
				$logger->add('payplus_save_token', 'Starting Saving Token');
			}
		}
		$token_num = wc_clean($res['token_uid']);
		$card_type = trim(wc_clean($res['brand_id']));
		$last_four = wc_clean($res['four_digits']);
		$exp_month = wc_clean($res['expiry_month']);
		$exp_year  = '20' . wc_clean($res['expiry_year']);

		$token = new WC_Payment_Token_CC();
		$token->set_token($token_num);
		$token->set_gateway_id($this->id);
		$token->set_card_type($card_type);
		$token->set_last4($last_four);
		$token->set_expiry_month($exp_month);
		$token->set_expiry_year($exp_year);
		$token->set_user_id($user_id ?: get_current_user_id());
		$token->save();
		if ($this->logging) {
			$logger->add('payplus_save_token', 'Saved And Finished: ' . print_r($token, true));
		}
	}

	// add credit card to my account
	public function add_payment_method()
	{
		if ($this->logging) {
			$logger = wc_get_logger();
			$logger->add('payplus_add_payment_method', 'New Add Payment Method Fired');
		}

		$current_user = wp_get_current_user();

		$langCode = explode("_", get_locale());
		$payload = '{
    "payment_page_uid": "' . $this->settings['payment_page_id'] . '",
	"charge_method": 5,
	"language_code": "' . $langCode[0] . '",
    "expiry_datetime": "30",
    "refURL_success": "' . $this->add_payment_res_url . '",
    "refURL_failure": "' . wc_get_endpoint_url('add-payment-method') . '",
    "refURL_callback": null,
    "customer": {
        "customer_name":"' . $current_user->display_name . '",
        "email":"' . $current_user->user_email . '"
    },
    "amount": ' . rand(1, 9) . ',
    "currency_code": "' . get_woocommerce_currency() . '",
    "sendEmailApproval": false,
    "sendEmailFailure": false,
    "create_token": true,
	"more_info": "WP Token ' . get_current_user_id() . '"
}';

		if ($this->logging) {
			$logger->add('payplus_add_payment_method', 'All data collected before Sending to PayPlus');
			$logger->add('payplus_add_payment_method', 'Request: ' . print_r($payload, true));
		}

		$response = $this->post_payplus_ws($this->payment_url, $payload);
		$res = json_decode(wp_remote_retrieve_body($response));
		if ($res->data->payment_page_link && $this->validateUrl($res->data->payment_page_link)) {
			if ($this->logging) {
				$logger->add('payplus_add_payment_method', 'WP Remote Post Completed');
				$logger->add('payplus_add_payment_method', 'Response: ' . print_r($res, true));
			}
			$this->get_payment_page($res->data->payment_page_link);
		} else {
			if ($this->logging) $logger->add('payplus_add_payment_method', 'WP Remote Post Error: ' . print_r($res, true));
			echo __('Something went wrong with the payment page') . '<hr /><b>Error:</b> ' . print_r($response['body'], true);
		}
	}

	// check add payment ipn response
	public function add_payment_ipn_response()
	{

		if ($this->logging) {
			$logger = wc_get_logger();
			$logger->add('payplus_add_payment_ipn', 'New Token Has Been Generated');
			$logger->add('payplus_add_payment_ipn', 'Response: ' . print_r($this->arr_clean($_REQUEST), true));
		}

		if (wc_clean($_REQUEST['type']) != "token" || wc_clean($_REQUEST['method']) != "credit-card") {
			if ($this->logging) $logger->add('payplus_add_payment_ipn', 'WS Error, No Token Type or Credit Card Method In Response');
			return;
		}

		if (wc_clean($_REQUEST['status']) == "approved" && wc_clean($_REQUEST['status_code']) == "000" && wc_clean($_REQUEST['token_uid'])) {
			wc_add_notice(__('Your new payment method has been added', 'payplus-payment-gateway'));
			if ($this->logging) $logger->add('payplus_add_payment_ipn', 'WS Success, Token has been generated: ' . print_r($this->arr_clean($_REQUEST), true));
			$this->save_token($_REQUEST, get_current_user_id());
		} else {
			wc_add_notice(__('There was a problem adding this card', 'payplus-payment-gateway'), 'error');
			if ($this->logging) $logger->add('payplus_add_payment_ipn', 'IPN Error: There was a problem adding this card');
		}


		wp_redirect(wc_get_endpoint_url('payment-methods', '', wc_get_page_permalink('myaccount')));
		exit;
	}

	public function scheduled_subscription_payment($amount_to_charge, $order)
	{
		$token = get_user_meta($order->user_id, 'cc_token', true);
		if ($this->logging) {
			$logger = wc_get_logger();
			$logger->add('payplus_subscription', 'Subscription Started. Order ID: ' . $order->id . ' - Token: ' . $token);
		}
		$result = $this->receipt_page($order->id, $token, true, 'WP_SUB_' . $order->id, $amount_to_charge);
		if ($this->logging) $logger->add('payplus_subscription', 'Subscription Response ' . $order->id . ':<br />' . print_r($result, true));

		if ($result->data->status == "approved" && $result->data->status_code == "000" && $result->data->transaction_uid) {
			$order->add_order_note(sprintf(__('PayPlus Subscription Payment Successful<br/>Transaction Number: %s', 'payplus-payment-gateway'), $result->data->transaction->number));
			if ($this->logging) $logger->add('payplus_subscription', 'Subscription ' . $order->id . ' Successful Response: ' . print_r($result, true));
			$order->update_status('completed');
			return ["success" => true, "msg" => ""];
		} else {
			$order->add_order_note(sprintf(__('PayPlus Subscription Payment Failure<br/>Transaction Number: %s', 'payplus-payment-gateway'), $result->data->transaction->number));
			if ($this->logging) $logger->add('payplus_subscription', 'Subscription ' . $order->id . ' Failure Response: ' . print_r($result, true));
			return ["success" => false, "msg" => "Authorization error: " . $result->data->status_code];
		}
	}

	public function arr_clean($req = [])
	{
		$REQUEST = array_map(function ($v) {
			return wc_clean($v);
		}, $req);
		return $REQUEST;
	}

	public function msg_checkout_code()
	{
		if ($this->api_test_mode) echo '<div style="background: #d23d3d; border-right: 8px #b33434 solid; border-radius: 4px; color: #FFF; padding: 5px;">' . __('Sandbox mode is active and real transaction cannot be processed. Please make sure to move production when finishing testing', 'payplus-payment-gateway') . '</div>';
	}
}